@@ -4,21 +4,18 @@ from i2.deco import (
     postprocess,
     preprocess_arguments,
     input_output_decorator,
+    wrap_class_methods_input_and_output,
+    double_up_as_factory,
 )
 
-from i2.deco import (
-    preprocess,
-    postprocess,
-    preprocess_arguments,
-    input_output_decorator,
-)
-from i2.deco import wrap_class_methods_input_and_output
 from i2.signatures import (
     Sig,
     Command,
     extract_commands,
     call_forgivingly,
     call_somewhat_forgivingly,
+    name_of_obj,
+    empty as empty_param_attr,
 )
 
 from i2.multi_object import (
@@ -30,3 +27,8 @@ from i2.multi_object import (
     ParallelFuncs,
     ContextFanout,
 )
+
+from i2.errors import (
+    InterruptWithBlock,
+    HandleExceptions,
+)
