@@ -1,22 +1,22 @@
 """Decorator tools"""
 import inspect
 from collections import defaultdict
-from collections.abc import Mapping
 from contextlib import suppress
-from functools import wraps, partial, update_wrapper
+from functools import wraps, partial
 from inspect import Signature, signature, Parameter
 from itertools import chain
-from typing import Iterable, Callable
+from typing import Callable
 
-from i2.signatures import ch_func_to_all_pk, Sig, tuple_the_args, kind_forgiving_func
+# from collections.abc import Mapping
+
+from i2.signatures import Sig, kind_forgiving_func
 
 # keep the imports below here because might be referenced (or take care of refs)
-from i2.signatures import ch_signature_to_all_pk, params_of, copy_func
+# from i2.signatures import ch_signature_to_all_pk, params_of, copy_func
 
 
 def double_up_as_factory(decorator_func):
     """Repurpose a decorator both as it's original form, and as a decorator factory.
-
     That is, from a decorator that is defined do ``wrapped_func = decorator(func, **params)``,
     make it also be able to do ``wrapped_func = decorator(**params)(func)``.
 
@@ -50,6 +50,23 @@ def double_up_as_factory(decorator_func):
     ...
     >>> foo(2)
     9
+
+    Note that to be able to use double_up_as_factory, your first argument (the object to be wrapped) needs to default
+    to None and be the only argument that is not keyword-only (i.e. all other arguments need to be keyword only).
+
+    >>> @double_up_as_factory
+    ... def decorator_2(func, *, multiplier=2):
+    ...     '''Should not be able to be transformed with double_up_as_factory'''
+    Traceback (most recent call last):
+      ...
+    AssertionError: First argument of the decorator function needs to default to None. Was <class 'inspect._empty'>
+    >>> @double_up_as_factory
+    ... def decorator_3(func=None, multiplier=2):
+    ...     '''Should not be able to be transformed with double_up_as_factory'''
+    Traceback (most recent call last):
+      ...
+    AssertionError: All arguments (besides the first) need to be keyword-only
+
     """
 
     def validate_decorator_func(decorator_func):
